//
//  Includes
//
#include <Trade\DealInfo.mqh>
#include <Trade\OrderInfo.mqh>
#include <Trade\SymbolInfo.mqh>
#include <Trade\Trade.mqh>

//
//  Globals
//
input group "General settings";
input int MagicNumber = 166870; // Magic number (unique number per EA, per instance)
input double PipPointOverride = 0; // Manual pip point (0 = auto detection)
input int MaxDeviationSlippage = 10; // Maximum deviation/ slippage
input string OrderComment = "MetaQuotes MT5 (166870)"; // Order comment
uint OrderFillingType = -1;

//
//  MQL5 Library Functions
//
double Ask_LibFunc()
{
   MqlTick last_tick;
   SymbolInfoTick(_Symbol,last_tick);
   return last_tick.ask;
}

double Bid_LibFunc()
{
   MqlTick last_tick;
   SymbolInfoTick(_Symbol,last_tick);
   return last_tick.bid;
}

uint GetFillingType()
{
   uint fillingType = -1;

   uint filling = (uint)SymbolInfoInteger(Symbol(),SYMBOL_FILLING_MODE);
   if ((filling&SYMBOL_FILLING_FOK) == SYMBOL_FILLING_FOK)
   {
      fillingType = ORDER_FILLING_FOK;
      Print("Filling type: FOK");
   }
   else if ((filling&SYMBOL_FILLING_IOC) == SYMBOL_FILLING_IOC)
   {
      fillingType = ORDER_FILLING_IOC;
      Print("Filling type: IOC");
   }
   else
   {
      fillingType = ORDER_FILLING_RETURN;
      Print("Filling type: RETURN");
   }
   
   return fillingType;
}

uint GetExecutionType()
{
   // If it returns SYMBOL_TRADE_EXECUTION_INSTANT (or SYMBOL_TRADE_EXECUTION_REQUEST) then you can use "deviation" with OrderSend. Otherwise you can't.
   uint executionType = -1;

   uint execution=(uint)SymbolInfoInteger(Symbol(),SYMBOL_TRADE_EXEMODE);
   if((execution&SYMBOL_TRADE_EXECUTION_MARKET)==SYMBOL_TRADE_EXECUTION_MARKET)
   {
      executionType=SYMBOL_TRADE_EXECUTION_MARKET;
      Print("Deal execution mode: Market execution, deviation setting will be ignored.");
   }
   else if((execution&SYMBOL_TRADE_EXECUTION_INSTANT)==SYMBOL_TRADE_EXECUTION_INSTANT)
   {
      executionType=SYMBOL_TRADE_EXECUTION_INSTANT;
      Print("Deal execution mode: Instant execution, deviation setting might be taken into account, depending on your broker.");
   }
   else if((execution&SYMBOL_TRADE_EXECUTION_REQUEST)==SYMBOL_TRADE_EXECUTION_REQUEST)
   {
      executionType=SYMBOL_TRADE_EXECUTION_REQUEST;
      Print("Deal execution mode: Request execution, deviation setting might be taken into account, depending on your broker.");
   }
   else if((execution&SYMBOL_TRADE_EXECUTION_EXCHANGE)==SYMBOL_TRADE_EXECUTION_EXCHANGE)
   {
      executionType=SYMBOL_TRADE_EXECUTION_EXCHANGE;
      Print("Deal execution mode: Exchange execution, deviation setting will be ignored.");
   }
   
   return executionType;
}

uint GetAccountMarginMode()
{
   uint marginMode = -1;

   marginMode = (ENUM_ACCOUNT_MARGIN_MODE)AccountInfoInteger(ACCOUNT_MARGIN_MODE);
   if (marginMode == ACCOUNT_MARGIN_MODE_RETAIL_NETTING)
   {
      Print("Account margin mode: Netting");
   }
   else if (marginMode == ACCOUNT_MARGIN_MODE_RETAIL_HEDGING)
   {
      Print("Account margin mode: Hedging");
   }
   else if (marginMode == ACCOUNT_MARGIN_MODE_EXCHANGE)
   {
      Print("Account margin mode: Exchange");
   }
   else
   {
      Print("Unkown margin type");
   }

   return marginMode;
}

// Pip Point Function
double GetRealPipPoint(string Currency)
{
	double calcPoint = 0;
	double calcDigits = Digits();
	Print("Number of digits after decimal point: " + DoubleToString(calcDigits));
 
	if (calcDigits == 0)
	{
		calcPoint = 1;
	}
	else if (calcDigits == 1)
	{
		calcPoint = 1;
	}
	else if(calcDigits == 2)
	{
	   calcPoint = 0.1;
	}
	else if (calcDigits == 3) 
	{
		calcPoint = 0.01;
    }
	else if(calcDigits == 4 || calcDigits == 5) 
		calcPoint = 0.0001;

	return(calcPoint);
}

//
//  Order
//
class Order
{
	public:
		ulong Ticket;
		double Lots;
		bool IsOpen;
		bool IsWaitingForDealExecution;

		void Order()
		{
		}

		void Order(Order* order, bool mustBeVisibleOnChart)
		{
			Ticket = order.Ticket;
			Lots = order.Lots;
   	}
};

//
//  OrderRepository
//
class OrderRepository
{
   public:
      static ulong OpenOrder(Order* order)
      {
         ulong orderTicket = ULONG_MAX;

         MqlTradeRequest request = {};
         MqlTradeResult result = {};
         
         request.action         = TRADE_ACTION_DEAL;
         request.symbol         = Symbol();
         request.volume         = order.Lots;
         request.type           = ORDER_TYPE_BUY;
         request.price          = Ask_LibFunc();
         request.magic          = MagicNumber;
         request.type_filling   = (ENUM_ORDER_TYPE_FILLING)OrderFillingType;
         
         ResetLastError();

         if(OrderSend(request, result))
         {
            if (result.retcode == TRADE_RETCODE_DONE || result.retcode == TRADE_RETCODE_PLACED)
            {
               orderTicket = result.order;
               order.IsWaitingForDealExecution = true;
               order.Ticket = orderTicket;
            }
            else
            {
               Print(StringFormat("OrderSend: retcode=%u", result.retcode));
            }
         }
         else
         {
            Print(StringFormat("OrderSend: error %d: %s", GetLastError(), "GetDescription(result.retcode)"));
         }
         
         return orderTicket != ULONG_MAX;
      }
      
      static bool ClosePosition(Order* order)
      {
         CPositionInfo  m_position;
         CTrade         m_trade;
      
         bool foundPosition = false;
         for(int i = PositionsTotal() -1; i >= 0; i--)
         {
            if(m_position.SelectByIndex(i))
            {
               if(m_position.Ticket() == order.Ticket) // On a hedging account the open order ticket and the position ticket are the same number
               {
                  foundPosition = true;
                  uint returnCode = 0;

                  if (m_trade.PositionClosePartial(order.Ticket, NormalizeDouble(order.Lots, 2), MaxDeviationSlippage))
                  {
                     returnCode = m_trade.ResultRetcode();
                     if (returnCode == TRADE_RETCODE_DONE || returnCode == TRADE_RETCODE_PLACED)
                     {
                        ulong orderTicket = m_trade.ResultOrder();
                        order.Ticket = orderTicket; // the close order gets a new order ticket, so it must be updated

                        Print(StringFormat("Successfully created a close order (%d) by EA (%d). Awaiting execution.", orderTicket, MagicNumber));
                        return true;                     
                     }

                     Print(StringFormat("Placing close order failed, Return code: %d", returnCode));
                  }
               }
            }     
         }

         return false;
      }
};



//
//  Expert Advisor Class
//
class EA
{
    public:
        Order* Order;
        bool EAIsFinished;
        
        void EA()
        {
            Order = NULL;
        }
        
        void HandleTick()
        {
            if (EAIsFinished)
               return;
        
            if (Order == NULL)
            {
               Order = new Order();
               Order.Lots = 0.01;
               if (!OpenOrder(Order))
               {
                  // Failed, market closed etc.
                  Order = NULL;
               }
            }
            else if (Order != NULL && !Order.IsWaitingForDealExecution)
            {
               ClosePosition(Order);
            }
        }
        
        bool OpenOrder(Order* order)
        {
            //..
            bool success = OrderRepository::OpenOrder(order);
            //..
            return success;
        }
           
        bool ClosePosition(Order* order)
        {
            //..
            bool success = OrderRepository::ClosePosition(order);
            //..
            return success;
        }
};

EA *_ea;

//+------------------------------------------------------------------------+
//|  Expert Initialization Function                                        |
//+------------------------------------------------------------------------+

int OnInit()
{
	
	OrderFillingType = GetFillingType();
	if ((int)OrderFillingType == -1)
	{
	    return (INIT_FAILED);
	}
	
	GetExecutionType();
	
	_ea = new EA();
	
	return (INIT_SUCCEEDED);
	
}

//+------------------------------------------------------------------------+
//|  Expert OnTransaction Function                                         |
//+------------------------------------------------------------------------+

void OnTradeTransaction(const MqlTradeTransaction& trans, const MqlTradeRequest& request, const MqlTradeResult& result)
{
	
	switch(trans.type)
	{
	    case TRADE_TRANSACTION_DEAL_ADD:
	    {
	        // Load deals from previous day in cache
	        datetime end = TimeCurrent();      
	        datetime start = end - PeriodSeconds(PERIOD_D1); 
	        HistorySelect(start, end + PeriodSeconds(PERIOD_D1));
	         
	        int dealsTotal = HistoryDealsTotal();
	        if (dealsTotal == 0)
	        {
	            Print("No deals found");
	            return;
	        }
	
	        ulong orderId = HistoryDealGetInteger(trans.deal, DEAL_ORDER);
	        if (orderId == 0)
	        {
	            Print("POSSIBLE BUG, order not found based on deal with transaction.deal #" + IntegerToString(trans.deal) + ", resulting Id == 0.");
	        }
	
	        CDealInfo dealInfo;
	        dealInfo.Ticket(trans.deal);
	        
	        ENUM_DEAL_ENTRY deal_entry = dealInfo.Entry();
	        
	        Print("The entry type of this deal is ");
	        if (deal_entry == DEAL_ENTRY_IN) // entry in is for updating pendingopen orders
	        {
	            if (_ea.Order.IsOpen)
	            {
	               Print("Possible bug, we're receiving an DEAL_ENTRY_IN, DEAL_ENTRY_OUT was expected");
	            }
	        
	            Print("DEAL_ENTRY_IN");
	            _ea.Order.IsWaitingForDealExecution = false;
	        }
	        // entry out is for updating pendingclose orders
	        else if (deal_entry == DEAL_ENTRY_OUT)
	        {
	            Print("DEAL_ENTRY_OUT");
	            _ea.Order.IsWaitingForDealExecution = false;
	            
	            Print("Test finished.");
	            _ea.EAIsFinished = true;
	        }
	     }
	  }
}

//+------------------------------------------------------------------------+
//|  Expert Deinitialization Function                                      |
//+------------------------------------------------------------------------+

void OnDeinit(const int reason)
{
	delete(_ea);
}

//+------------------------------------------------------------------------+
//|  Expert Advisor Function                                               |
//+------------------------------------------------------------------------+

void OnTick()
{
	_ea.HandleTick();
}


